#
# Copyright (c) 2003 Gilles Lenfant
# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""This is the EpozDocument Product for Zope
An EpozDocument is a content object with Epoz edition capability
$Id: EpozDocument.py,v 1.1.1.1 2003/10/24 16:51:07 zopecvs Exp $
"""

__version__ = "$Revision: 1.1.1.1 $"[11:-2]

from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from Globals import DTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import change_images_and_files
from DateTime import DateTime
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface


################
## Main class ##
################

class EpozDocument(CatalogAware, PropertyManager, SimpleItem):
    """An EpozDocument is a content object with Epoz edition capability
    """
    __implements__ = (WriteLockInterface,)

    meta_type = 'EpozDocument'

    _properties = (
        {'id': 'title', 'type': 'string', 'mode': 'w'},
        {'id': 'EpozDocumentTemplate', 'type': 'string', 'mode': 'w'}
        )

    manage_options = ({'label': 'Edit', 'action': 'manage_editForm'},
                      {'label': 'View', 'action': ''}
                      ) + PropertyManager.manage_options + SimpleItem.manage_options

    security = ClassSecurityInfo()

    ##################
    ## User methods ##
    ##################

    def __init__(self, id, title=''):
        """
        """
        self._internalVersion = 1 # Provision for future upgrades
        self.id = id
        self.title = title
        self.EpozDocumentTemplate = ''
        self.html = ''
        return

    # Prevent accidental acquisition
    index_html = None

    security.declarePublic('__call__')
    def __call__(self, client=None, REQUEST=None, RESPONSE=None, **kw):
        """Publishing (through the EpozDocumentTemplate if any)
        """
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        templateId = 'EpozDocumentTemplate'
        templateId = getattr(self.aq_base, templateId) or templateId
        template = getattr(self.aq_parent, templateId, None)
        if template:
            template = template.__of__(self)
            return apply(template, ((client, self), REQUEST), kw)

        # Publish directly as HTML
        html = "<html><head><title>%s</title></head><body>%s</body></html>"
        html = html % (self.title, self.html)
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE
        RESPONSE.setHeader('Content-Type', 'text/html')
        RESPONSE.setHeader('Content-Length', str(len(html)))
        return html

    security.declarePublic('SearchableText')
    def SearchableText(self):
        """Provides all text for indexing
        """
        extrasId = 'SearchableTextAdd'
        textTypes = ('string', 'text')
        extraText = ''
        if self.hasProperty(extrasId):
            for extra in self.getProperty(extrasId):
                if (self.hasProperty(extra) and
                    (self.getPropertyType(extra) in textTypes)):
                    extraText += self.getProperty(extra) + ' '
        return " ".join((self.title, self.html, extraText))

    PrincipiaSearchSource = SearchableText


    #################
    ## ZMI methods ##
    #################

    manage_editForm = PageTemplateFile('zmi/manage_editForm', globals())

    security.declareProtected(change_images_and_files, 'manage_edit')
    def manage_edit(self, title, EpozDocumentTemplate, html, REQUEST=None):
        """Edit the stuff (from ZMI)
        """
        if self.wl_isLocked():
            raise ResourceLockedError, "EpozDocument is locked via WebDAV"
        self.title = title.strip()
        self.EpozDocumentTemplate = EpozDocumentTemplate
        self.html = html
        self.reindex_object()
        if REQUEST:
            message="Saved changes."
            return self.manage_editForm(self,REQUEST,manage_tabs_message=message,
                                        management_view='Edit')
        return

    security.declareProtected(change_images_and_files, 'manage_widgetdims')
    def manage_widgetdims(self, REQUEST):
        """Change dimensions of widget and record dimensions in cookies
        """
        change = REQUEST.form['manage_widgetdims']
        rules = {
            'Taller': ('__epozfile_height__', 200, 50),
            'Shorter': ('__epozfile_height__', 200, -50),
            'Wider': ('__epozfile_width__', 400, 50),
            'Narrower': ('__epozfile_width__', 400, -50)}
        cookieName, defaultSize, add = rules[change]
        expires = (DateTime('GMT') + 365).rfc822()
        size = REQUEST.get(cookieName, str(defaultSize))
        size = str(int(size) + add)
        # Makes change global for all EpozDocument objects of this Zope
        REQUEST.RESPONSE.setCookie(cookieName, size, path='/', expires=expires)
        message="Widget %s of %d pixels" % (change.lower(), abs(add))
        return self.manage_editForm(self, REQUEST, manage_tabs_message=message,
                                    management_view='Epoz')

    ####################
    ## Common methods ##
    ####################

    security.declarePublic('get_size')
    def get_size(self):
        """The size of the content
        """
        return len(self.html)

    # Support for deprecated
    getSize = get_size

    ########################################
    ## FTP/WebDav/External Editor support ##
    ########################################

    security.declareProtected(change_images_and_files, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests
        """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        self.html = REQUEST.get('BODY', '')
        self.reindex_object()
        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(change_images_and_files, 'manage_FTPget')
    def manage_FTPget(self):
        """Get source for FTP download
        """
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/html')
        return self.html


    #####################
    ## Private methods ##
    #####################

    def _postInit(self):
        """Continues __init__(...) after setting in ZODB
        """
        self.index_object()
        return

###############################
## Zope factory registration ##
###############################

def initialize(context):
    """Registers this Product into Zope
    """
    context.registerClass(
        instance_class = EpozDocument,
        permission='Add EpozDocument',
        constructors=(manage_addEpozDocumentForm, manage_addEpozDocument),
        icon='www/EpozDocument_icon.gif')
    return

########################
## Zope factory hooks ##
########################

def manage_addEpozDocument(container, id, title='', REQUEST=None):
    """Creates an EpozDocument in the container
    """
    id = id.strip()
    o = EpozDocument(id, title=title)
    container._setObject(id, o)
    o._postInit()
    if REQUEST:
        if REQUEST.form.has_key("add"):
            nextPath = '/manage_workspace'
        else:
            # "addedit" button clicked
            nextPath = '/%s/manage_workspace' % id
        REQUEST['RESPONSE'].redirect(container.absolute_url() + nextPath)
    return

manage_addEpozDocumentForm = DTMLFile('zmi/addEpozDocumentForm', globals())
